// Copyright (c) 2018 The Bitcoin Core developers
// Distributed under the MIT software license, see the accompanying
// file COPYING or http://www.opensource.org/licenses/mit-license.php.

#ifndef BITCOIN_INTERFACES_NODE_H
#define BITCOIN_INTERFACES_NODE_H

#include <init.h>       // For HelpMessageMode
#include <netaddress.h> // For Network

#include <functional>
#include <memory>
#include <string>

class Config;
class HTTPRPCRequestProcessor;
class proxyType;
class RPCServer;

namespace interfaces {

class Handler;

//! Top-level interface for a bitcoin node (bitcoind process).
class Node {
public:
    virtual ~Node() {}

    //! Set command line arguments.
    virtual void parseParameters(int argc, const char *const argv[]) = 0;

    //! Set a command line argument if it doesn't already have a value
    virtual bool softSetArg(const std::string &arg,
                            const std::string &value) = 0;

    //! Set a command line boolean argument if it doesn't already have a value
    virtual bool softSetBoolArg(const std::string &arg, bool value) = 0;

    //! Load settings from configuration file.
    virtual void readConfigFile(const std::string &conf_path) = 0;

    //! Choose network parameters.
    virtual void selectParams(const std::string &network) = 0;

    //! Init logging.
    virtual void initLogging() = 0;

    //! Init parameter interaction.
    virtual void initParameterInteraction() = 0;

    //! Get warnings.
    virtual std::string getWarnings(const std::string &type) = 0;

    //! Initialize app dependencies.
    virtual bool baseInitialize(Config &config, RPCServer &rpcServer) = 0;

    //! Start node.
    virtual bool
    appInitMain(Config &config,
                HTTPRPCRequestProcessor &httpRPCRequestProcessor) = 0;

    //! Stop node.
    virtual void appShutdown() = 0;

    //! Start shutdown.
    virtual void startShutdown() = 0;

    //! Return whether shutdown was requested.
    virtual bool shutdownRequested() = 0;

    //! Get help message string.
    virtual std::string helpMessage(HelpMessageMode mode) = 0;

    //! Map port.
    virtual void mapPort(bool use_upnp) = 0;

    //! Get proxy.
    virtual bool getProxy(Network net, proxyType &proxy_info) = 0;

    //! Register handler for init messages.
    using InitMessageFn = std::function<void(const std::string &message)>;
    virtual std::unique_ptr<Handler> handleInitMessage(InitMessageFn fn) = 0;

    //! Register handler for message box messages.
    using MessageBoxFn =
        std::function<bool(const std::string &message,
                           const std::string &caption, unsigned int style)>;
    virtual std::unique_ptr<Handler> handleMessageBox(MessageBoxFn fn) = 0;

    //! Register handler for question messages.
    using QuestionFn = std::function<bool(
        const std::string &message, const std::string &non_interactive_message,
        const std::string &caption, unsigned int style)>;
    virtual std::unique_ptr<Handler> handleQuestion(QuestionFn fn) = 0;
};

//! Return implementation of Node interface.
std::unique_ptr<Node> MakeNode();

} // namespace interfaces

#endif // BITCOIN_INTERFACES_NODE_H
